from payload_authenticator import PayloadAuthenticator
from user_authenticator import UserAuthenticator
from rest_framework import status
from rest_framework.response import Response

'''
Guardian ..
'''


def guardian(*authenticators, **payload):
    def authenticate(func):
        def wrap(context, request):
            # gets the final authentication and request(if changed)
            is_authenticated, result = UserAuthenticator(authenticators, request).authenticate()

            # if the result is not authenticated then return 401
            if not is_authenticated:
                return Response({
                    'message': 'user is not authorized to access the url'
                }, status=status.HTTP_401_UNAUTHORIZED)

            # The UserAuthenticator might return None because the user returned it. Hence to verify whether it was
            # returned by the user or by me i am using a null string.
            # Cause if the return is 'null' then the user does not changed the request object and hence it does not
            # need to be re assigned
            if type(result) is str and result == 'null':
                pass
            else:
                request = result

            # Verifying the payload. If its verified then calling the view function else returning a 400
            if PayloadAuthenticator(payload, request).authenticate():
                return func(context, request)
            else:
                return Response({
                    'message': 'invalid payload'
                }, status=status.HTTP_400_BAD_REQUEST)

        wrap.__name__ = func.__name__
        return wrap

    return authenticate



'''
The file verifies the request payload.
'''


class PayloadAuthenticator:
    def __init__(self, payload, request):
        self.payload = payload
        self.request = request
        self.is_authenticated = True
        self.current_method = None
        self.request_data = None

    def authenticate(self):
        try:
            if not self.payload:
                return self.is_authenticated

            # using query params if the method is get
            if self.request.method == 'GET':
                self.current_method = 'GET'
                self.request_data = self.request.query_params
            else:
                self.current_method = 'POST'
                self.request_data = self.request.data

            self.__authenticator()
            return self.is_authenticated
        except:
            return False

    def __authenticator(self):
        for variable in self.payload:
            if variable not in self.request_data:
                self.is_authenticated = False
                break

            if not self.__type_verifier(variable):
                self.is_authenticated = False
                break

    def __type_verifier(self, variable):
        # if the variable is None then no need to check type
        if self.payload[variable] is None:
            return True

        # if the variable value is tuple then the data should be in the tuple
        elif type(self.payload[variable]) is tuple:
            return self.request_data[variable] in self.payload[variable]

        else:
            # checking the type of the variable if it matches the data type
            if self.current_method == 'GET' \
                    and type(self.payload[variable](self.request_data[variable])) == self.payload[variable]:
                return True
            elif self.current_method == 'POST' and type(self.request_data[variable]) == self.payload[variable]:
                return True

            return False



'''
The file authenticates the user depending on the guard method implemented by the user.
'''

import inspect


class UserAuthenticator(object):
    def __init__(self, authenticators, request):
        self.authenticator_list = authenticators
        self.is_authenticated = False
        self.request = request
        self.result = 'null'

    def authenticate(self):
        # if there is no authenticators return True
        if not self.authenticator_list:
            self.is_authenticated = True
            return self.is_authenticated, self.result

        for authenticator in self.authenticator_list:

            # Expecting authenticators to be of class type
            if not inspect.isclass(authenticator):
                return TypeError('Expected {0} to be of type class'.format(authenticator.__name__))

            # Expecting authenticator class to have implemented the guard method
            if not hasattr(authenticator, 'guard'):
                raise NotImplementedError('Guard method is not implemented in {0} class'.format(authenticator.__name__))

            # calling the guard method with request parameter
            response = authenticator().guard(self.request)

            # if the response is only boolean then check the response.
            # If its true then the user is verified
            if type(response) is bool:
                if response:
                    self.is_authenticated = True
                    break

            # if response is a tuple then the expectation are to be of form (True, request)
            elif type(response) is tuple:
                if len(response) > 2:
                    raise AssertionError('Too many values to unpack. ' +
                                         'Expected 2 found more from {0} class'.format(authenticator.__name__))
                if type(response[0]) is not bool:
                    raise TypeError('Expected boolean from {0} class'.format(authenticator.__name__))

                if response[0]:
                    self.is_authenticated = True
                    self.result = response[1]
                    # raising warning if the user return none in the request
                    if not response[1]:
                        raise Warning('Received null response from {0} class. This response will be passed to the '
                                      'view layer.'.format(authenticator.__name__))
                    break
            else:
                raise TypeError('Unexpected type return from {0} class'.format(authenticator.__name__))

        return self.is_authenticated, self.result



from setuptools import setup

setup(
    # Application name:
    name="django_rest_guardian",

    # Version number (initial):
    version="1.0.0",

    # Application author details:
    author="Ankur Jain",
    author_email="ankurj630@gmail.com",

    # Packages
    packages=["django_rest_guardian"],

    # Include additional files into the package
    # include_package_data=True,

    # Details
    url="https://github.com/droidlife/guardian",

    #
    # license="LICENSE.txt",
    description="Useful towel-related stuff.",
    install_requires=['djangorestframework']

    # long_description=open("README.txt").read(),
)
